@@ -23,12 +23,12 @@
 #define ast_register_flag_constant(name, value) \
 	REGISTER_NS_LONG_CONSTANT("ast\\flags", name, value, CONST_CS | CONST_PERSISTENT)
 
-#define AST_CACHE_SLOT_KIND     &AST_G(cache_slots)[2 * 0]
-#define AST_CACHE_SLOT_FLAGS    &AST_G(cache_slots)[2 * 1]
-#define AST_CACHE_SLOT_LINENO   &AST_G(cache_slots)[2 * 2]
-#define AST_CACHE_SLOT_CHILDREN &AST_G(cache_slots)[2 * 3]
+#define AST_CACHE_SLOT_KIND     &AST_G(cache_slots)[3 * 0]
+#define AST_CACHE_SLOT_FLAGS    &AST_G(cache_slots)[3 * 1]
+#define AST_CACHE_SLOT_LINENO   &AST_G(cache_slots)[3 * 2]
+#define AST_CACHE_SLOT_CHILDREN &AST_G(cache_slots)[3 * 3]
 
-#define AST_CURRENT_VERSION 50
+#define AST_CURRENT_VERSION 70
 
 /* Additional flags for BINARY_OP */
 #define AST_BINARY_IS_GREATER 256
@@ -42,7 +42,7 @@
 #define AST_PLUS 261
 #define AST_MINUS 262
 
-/* Define some constants for PHP 7.0 */
+/* Define some compatibility constants */
 #if PHP_VERSION_ID < 70100
 # define IS_VOID 18
 # define IS_ITERABLE 19
@@ -52,6 +52,15 @@
 # define ZEND_ARRAY_SYNTAX_SHORT 3
 #endif
 
+#if PHP_VERSION_ID < 70300
+# define ZEND_BIND_REF 1
+#endif
+
+#if PHP_VERSION_ID < 70400
+# define ZEND_DIM_ALTERNATIVE_SYNTAX (1<<1)
+# define ZEND_PARENTHESIZED_CONDITIONAL 1
+#endif
+
 /* This contains state of the ast Node creator. */
 typedef struct ast_state_info {
 	zend_long version;
@@ -60,14 +69,15 @@ typedef struct ast_state_info {
 
 typedef struct _ast_flag_info {
 	uint16_t ast_kind;
-	const char **flags;
 	zend_bool combinable;
+	const char **flags;
 } ast_flag_info;
 
 ZEND_DECLARE_MODULE_GLOBALS(ast)
 
+ast_str_globals str_globals;
+
 static zend_class_entry *ast_node_ce;
-static zend_class_entry *ast_decl_ce;
 static zend_class_entry *ast_metadata_ce;
 
 #define AST_FLAG(name) "ast\\flags\\" #name
@@ -129,7 +139,8 @@ static const char *unary_op_flags[] = {
 	AST_FLAG(BINARY_MOD), \
 	AST_FLAG(BINARY_POW), \
 	AST_FLAG(BINARY_SHIFT_LEFT), \
-	AST_FLAG(BINARY_SHIFT_RIGHT) \
+	AST_FLAG(BINARY_SHIFT_RIGHT), \
+	AST_FLAG(BINARY_COALESCE) \
 
 static const char *binary_op_flags[] = {
 	AST_SHARED_BINARY_OP_FLAGS,
@@ -145,7 +156,6 @@ static const char *binary_op_flags[] = {
 	AST_FLAG(BINARY_IS_GREATER),
 	AST_FLAG(BINARY_IS_GREATER_OR_EQUAL),
 	AST_FLAG(BINARY_SPACESHIP),
-	AST_FLAG(BINARY_COALESCE),
 	NULL
 };
 
@@ -189,6 +199,11 @@ static const char *array_flags[] = {
 	NULL
 };
 
+static const char *array_elem_flags[] = {
+	AST_FLAG(ARRAY_ELEM_REF),
+	NULL
+};
+
 static const char *closure_use_flags[] = {
 	AST_FLAG(CLOSURE_USE_REF),
 	NULL
@@ -222,35 +237,55 @@ static const char *func_flags[] = {
 	NULL
 };
 
+// Flags of AST_DIM are marked as combinable in case any other flags get added in the future.
+static const char *dim_flags[] = {
+	AST_FLAG(DIM_ALTERNATIVE_SYNTAX),
+	NULL
+};
+
+// Flags of AST_CONDITIONAL are marked as combinable in case any other flags get added in the future.
+static const char *conditional_flags[] = {
+	AST_FLAG(PARENTHESIZED_CONDITIONAL),
+	NULL
+};
+
 static const ast_flag_info flag_info[] = {
-	{ AST_NAME, name_flags, 0 },
-	{ ZEND_AST_CLASS, class_flags, 0 },
-	{ ZEND_AST_PARAM, param_flags, 1 },
-	{ ZEND_AST_TYPE, type_flags, 0 },
-	{ ZEND_AST_CAST, type_flags, 0 },
-	{ ZEND_AST_UNARY_OP, unary_op_flags, 0 },
-	{ ZEND_AST_BINARY_OP, binary_op_flags, 0 },
-	{ ZEND_AST_ASSIGN_OP, assign_op_flags, 0 },
-	{ ZEND_AST_MAGIC_CONST, magic_const_flags, 0 },
-	{ ZEND_AST_USE, use_flags, 0 },
-	{ ZEND_AST_GROUP_USE, use_flags, 0 },
-	{ ZEND_AST_USE_ELEM, use_flags, 0 },
-	{ ZEND_AST_INCLUDE_OR_EVAL, include_flags, 0 },
-	{ ZEND_AST_ARRAY, array_flags, 0 },
-	{ AST_CLOSURE_VAR, closure_use_flags, 0 },
-	{ ZEND_AST_METHOD, func_flags, 1 },
-	{ ZEND_AST_FUNC_DECL, func_flags, 1 },
-	{ ZEND_AST_CLOSURE, func_flags, 1 },
-	{ ZEND_AST_PROP_DECL, modifier_flags, 1 },
-	{ ZEND_AST_CLASS_CONST_DECL, visibility_flags, 1 },
-	{ ZEND_AST_TRAIT_ALIAS, modifier_flags, 1 },
+	{ AST_NAME, 0, name_flags },
+	{ ZEND_AST_CLASS, 0, class_flags },
+	{ ZEND_AST_PARAM, 1, param_flags },
+	{ ZEND_AST_TYPE, 0, type_flags },
+	{ ZEND_AST_CAST, 0, type_flags },
+	{ ZEND_AST_UNARY_OP, 0, unary_op_flags },
+	{ ZEND_AST_BINARY_OP, 0, binary_op_flags },
+	{ ZEND_AST_ASSIGN_OP, 0, assign_op_flags },
+	{ ZEND_AST_MAGIC_CONST, 0, magic_const_flags },
+	{ ZEND_AST_USE, 0, use_flags },
+	{ ZEND_AST_GROUP_USE, 0, use_flags },
+	{ ZEND_AST_USE_ELEM, 0, use_flags },
+	{ ZEND_AST_INCLUDE_OR_EVAL, 0, include_flags },
+	{ ZEND_AST_ARRAY, 0, array_flags },
+	{ ZEND_AST_ARRAY_ELEM, 0, array_elem_flags },
+	{ AST_CLOSURE_VAR, 0, closure_use_flags },
+	{ ZEND_AST_METHOD, 1, func_flags },
+	{ ZEND_AST_FUNC_DECL, 1, func_flags },
+	{ ZEND_AST_CLOSURE, 1, func_flags },
+	{ ZEND_AST_ARROW_FUNC, 1, func_flags },
+	{ ZEND_AST_PROP_DECL, 1, modifier_flags },
+	{ ZEND_AST_PROP_GROUP, 1, modifier_flags },
+	{ ZEND_AST_CLASS_CONST_DECL, 1, visibility_flags },
+	{ ZEND_AST_TRAIT_ALIAS, 1, modifier_flags },
+	{ ZEND_AST_DIM, 1, dim_flags },
+	{ ZEND_AST_CONDITIONAL, 1, conditional_flags },
 };
 
 static inline void ast_update_property(zval *object, zend_string *name, zval *value, void **cache_slot) {
+#if PHP_VERSION_ID < 80000
 	zval name_zv;
 	ZVAL_STR(&name_zv, name);
-
 	Z_OBJ_HT_P(object)->write_property(object, &name_zv, value, cache_slot);
+#else
+	Z_OBJ_HT_P(object)->write_property(Z_OBJ_P(object), name, value, cache_slot);
+#endif
 }
 
 static inline void ast_update_property_long(zval *object, zend_string *name, zend_long value_raw, void **cache_slot) {
@@ -300,13 +335,15 @@ static inline zend_bool ast_kind_uses_attr(zend_ast_kind kind) {
 		|| kind == ZEND_AST_UNARY_OP || kind == ZEND_AST_BINARY_OP || kind == ZEND_AST_ASSIGN_OP
 		|| kind == ZEND_AST_CAST || kind == ZEND_AST_MAGIC_CONST || kind == ZEND_AST_ARRAY_ELEM
 		|| kind == ZEND_AST_INCLUDE_OR_EVAL || kind == ZEND_AST_USE || kind == ZEND_AST_PROP_DECL
+		|| kind == ZEND_AST_PROP_GROUP
 		|| kind == ZEND_AST_GROUP_USE || kind == ZEND_AST_USE_ELEM
 		|| kind == AST_NAME || kind == AST_CLOSURE_VAR || kind == ZEND_AST_CLASS_CONST_DECL
-		|| kind == ZEND_AST_ARRAY;
+		|| kind == ZEND_AST_ARRAY || kind == ZEND_AST_DIM || kind == ZEND_AST_CONDITIONAL;
 }
 
 static inline zend_bool ast_kind_is_decl(zend_ast_kind kind) {
 	return kind == ZEND_AST_FUNC_DECL || kind == ZEND_AST_CLOSURE
+		|| kind == ZEND_AST_ARROW_FUNC
 		|| kind == ZEND_AST_METHOD || kind == ZEND_AST_CLASS;
 }
 
@@ -325,6 +362,7 @@ static inline zend_bool ast_is_name(zend_ast *ast, zend_ast *parent, uint32_t i)
 			|| parent->kind == ZEND_AST_CALL || parent->kind == ZEND_AST_CONST
 			|| parent->kind == ZEND_AST_NEW || parent->kind == ZEND_AST_STATIC_CALL
 			|| parent->kind == ZEND_AST_CLASS_CONST || parent->kind == ZEND_AST_STATIC_PROP
+			|| parent->kind == ZEND_AST_PROP_GROUP || parent->kind == ZEND_AST_CLASS_NAME
 			;
 	}
 
@@ -334,6 +372,9 @@ static inline zend_bool ast_is_name(zend_ast *ast, zend_ast *parent, uint32_t i)
 
 	if (i == 3) {
 		return parent->kind == ZEND_AST_FUNC_DECL || parent->kind == ZEND_AST_CLOSURE
+#if PHP_VERSION_ID >= 70400
+			|| parent->kind == ZEND_AST_ARROW_FUNC
+#endif
 			|| parent->kind == ZEND_AST_METHOD;
 	}
 
@@ -343,10 +384,17 @@ static inline zend_bool ast_is_name(zend_ast *ast, zend_ast *parent, uint32_t i)
 /* Assumes that ast_is_name is already true */
 static inline zend_bool ast_is_type(zend_ast *ast, zend_ast *parent, uint32_t i) {
 	if (i == 0) {
-		return parent->kind == ZEND_AST_PARAM;
+		return parent->kind == ZEND_AST_PARAM
+#if PHP_VERSION_ID >= 70400
+			|| parent->kind == ZEND_AST_PROP_GROUP
+#endif
+			;
 	}
 	if (i == 3) {
 		return parent->kind == ZEND_AST_CLOSURE || parent->kind == ZEND_AST_FUNC_DECL
+#if PHP_VERSION_ID >= 70400
+			|| parent->kind == ZEND_AST_ARROW_FUNC
+#endif
 			|| parent->kind == ZEND_AST_METHOD;
 	}
 	return 0;
@@ -410,6 +458,7 @@ static inline zend_uchar lookup_builtin_type(const zend_string *name) {
 	return 0;
 }
 
+#if PHP_VERSION_ID < 70400
 static inline zend_ast_attr ast_assign_op_to_binary_op(zend_ast_attr attr) {
 	switch (attr) {
 		case ZEND_ASSIGN_BW_OR: return ZEND_BW_OR;
@@ -427,6 +476,7 @@ static inline zend_ast_attr ast_assign_op_to_binary_op(zend_ast_attr attr) {
 		EMPTY_SWITCH_DEFAULT_CASE()
 	}
 }
+#endif
 
 static inline zend_bool ast_array_is_list(zend_ast *ast) {
 	zend_ast_list *list = zend_ast_get_list(ast);
@@ -497,10 +547,50 @@ static void ast_create_virtual_node(
 		zval *zv, zend_ast_kind kind, zend_ast_attr attr, zend_ast *child, ast_state_info_t *state) {
 	zval child_zv;
 	ast_to_zval(&child_zv, child, state);
-	return ast_create_virtual_node_ex(
+	ast_create_virtual_node_ex(
 		zv, kind, attr, zend_ast_get_lineno(child), state, 1, &child_zv);
 }
 
+static inline void ast_name_to_zval(zend_ast *child, zend_ast *ast, zval *child_zv, int i, ast_state_info_t *state) {
+	zend_uchar type;
+	zend_bool is_nullable = 0;
+	if (child->attr & ZEND_TYPE_NULLABLE) {
+		is_nullable = 1;
+		child->attr &= ~ZEND_TYPE_NULLABLE;
+	}
+
+	if (child->attr == ZEND_NAME_FQ) {
+		/* Ensure there is no leading \ for fully-qualified names. This can happen if
+		 * something like ('\bar')() is used. */
+		zval *name = zend_ast_get_zval(child);
+		if (Z_STRVAL_P(name)[0] == '\\') {
+			zend_string *new_name = zend_string_init(
+				Z_STRVAL_P(name) + 1, Z_STRLEN_P(name) - 1, 0);
+			zend_string_release(Z_STR_P(name));
+			Z_STR_P(name) = new_name;
+		}
+	}
+
+	if (child->attr == ZEND_NAME_NOT_FQ
+			&& ast_is_type(child, ast, i)
+			&& (type = lookup_builtin_type(zend_ast_get_str(child)))
+	) {
+		/* Convert "int" etc typehints to TYPE nodes */
+		ast_create_virtual_node_ex(
+			child_zv, ZEND_AST_TYPE, type, zend_ast_get_lineno(child), state, 0);
+	} else {
+		ast_create_virtual_node(child_zv, AST_NAME, child->attr, child, state);
+	}
+
+	if (is_nullable) {
+		/* Create explicit AST_NULLABLE_TYPE node */
+		zval tmp;
+		ZVAL_COPY_VALUE(&tmp, child_zv);
+		ast_create_virtual_node_ex(
+			child_zv, AST_NULLABLE_TYPE, 0, zend_ast_get_lineno(child), state, 1, &tmp);
+	}
+}
+
 static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t *state) {
 	uint32_t i, count;
 	zend_bool is_list = zend_ast_is_list(ast);
@@ -515,68 +605,23 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 				ast_fill_children_ht(ht, child, state);
 				continue;
 			}
-			if (state->version >= 40 && child == NULL) {
+			if (child == NULL) {
 				continue;
 			}
 		}
 
-		/* These two AST_CATCH checks should occur before ast_is_name() */
-#if PHP_VERSION_ID >= 70100
-		if (ast->kind == ZEND_AST_CATCH && state->version < 35
-				&& i == 0 && zend_ast_get_list(child)->children == 1) {
-			/* Emulate PHP 7.0 format (no list) */
-			zend_ast *first_child = zend_ast_get_list(child)->child[0];
-			ast_create_virtual_node(
-				&child_zv, AST_NAME, first_child->attr, first_child, state);
-		}
-#else
-		if (ast->kind == ZEND_AST_CATCH && state->version >= 35 && i == 0) {
+		/* This AST_CATCH check should occur before ast_is_name() */
+#if PHP_VERSION_ID < 70100
+		if (ast->kind == ZEND_AST_CATCH && i == 0) {
 			/* Emulate PHP 7.1 format (name list) */
 			zval tmp;
 			ast_create_virtual_node(&tmp, AST_NAME, child->attr, child, state);
 			ast_create_virtual_node_ex(
 				&child_zv, ZEND_AST_NAME_LIST, 0, zend_ast_get_lineno(child), state, 1, &tmp);
-		}
+		} else
 #endif
-		else if (ast_is_name(child, ast, i)) {
-			zend_uchar type;
-			zend_bool is_nullable = 0;
-			if (child->attr & ZEND_TYPE_NULLABLE) {
-				is_nullable = 1;
-				child->attr &= ~ZEND_TYPE_NULLABLE;
-			}
-
-			if (state->version >= 40 && child->attr == ZEND_NAME_FQ) {
-				/* Ensure there is no leading \ for fully-qualified names. This can happen if
-				 * something like ('\bar')() is used. */
-				zval *name = zend_ast_get_zval(child);
-				if (Z_STRVAL_P(name)[0] == '\\') {
-					zend_string *new_name = zend_string_init(
-						Z_STRVAL_P(name) + 1, Z_STRLEN_P(name) - 1, 0);
-					zend_string_release(Z_STR_P(name));
-					Z_STR_P(name) = new_name;
-				}
-			}
-
-			if (state->version >= 40 && child->attr == ZEND_NAME_NOT_FQ
-					&& ast_is_type(child, ast, i)
-					&& (type = lookup_builtin_type(zend_ast_get_str(child)))
-					&& (type != IS_OBJECT || state->version >= 45)
-			) {
-				/* Convert "int" etc typehints to TYPE nodes */
-				ast_create_virtual_node_ex(
-					&child_zv, ZEND_AST_TYPE, type, zend_ast_get_lineno(child), state, 0);
-			} else {
-				ast_create_virtual_node(&child_zv, AST_NAME, child->attr, child, state);
-			}
-
-			if (is_nullable) {
-				/* Create explicit AST_NULLABLE_TYPE node */
-				zval tmp;
-				ZVAL_COPY_VALUE(&tmp, &child_zv);
-				ast_create_virtual_node_ex(
-					&child_zv, AST_NULLABLE_TYPE, 0, zend_ast_get_lineno(child), state, 1, &tmp);
-			}
+		if (ast_is_name(child, ast, i)) {
+			ast_name_to_zval(child, ast, &child_zv, i, state);
 		} else if (child && child->kind == ZEND_AST_TYPE && (child->attr & ZEND_TYPE_NULLABLE)) {
 			child->attr &= ~ZEND_TYPE_NULLABLE;
 			ast_create_virtual_node(&child_zv, AST_NULLABLE_TYPE, 0, child, state);
@@ -584,7 +629,7 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 			ast_create_virtual_node(&child_zv, AST_CLOSURE_VAR, child->attr, child, state);
 		} else if (ast_is_var_name(child, ast, i)) {
 			ast_create_virtual_node(&child_zv, ZEND_AST_VAR, 0, child, state);
-		} else if (state->version >= 40 && ast_should_normalize_list(child, ast, i)) {
+		} else if (ast_should_normalize_list(child, ast, i)) {
 			if (child) {
 				zval tmp;
 				ast_to_zval(&tmp, child, state);
@@ -594,16 +639,21 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 				ast_create_virtual_node_ex(
 					&child_zv, ZEND_AST_STMT_LIST, 0, zend_ast_get_lineno(ast), state, 0);
 			}
-		} else if (state->version < 50 && i == 2
-				&& (ast->kind == ZEND_AST_PROP_ELEM || ast->kind == ZEND_AST_CONST_ELEM)) {
-			/* Skip docComment child -- It's handled separately */
+		} else if (state->version >= 60 && i == 1
+				&& (ast->kind == ZEND_AST_FUNC_DECL || ast->kind == ZEND_AST_METHOD)) {
+			/* Skip "uses" child, it is only relevant for closures */
 			continue;
+#if PHP_VERSION_ID >= 70400
+		} else if (i == 1 && ast->kind == ZEND_AST_ARROW_FUNC) {
+			/* Skip "uses" child since it is always empty */
+			continue;
+#endif
 #if PHP_VERSION_ID >= 70100
 		} else if (ast->kind == ZEND_AST_LIST && child != NULL) {
 			/* Emulate simple variable list */
 			ast_to_zval(&child_zv, child->child[0], state);
 #else
-		} else if (state->version >= 35 && ast->kind == ZEND_AST_ARRAY
+		} else if (ast->kind == ZEND_AST_ARRAY
 				&& ast->attr == ZEND_ARRAY_SYNTAX_LIST && child != NULL) {
 			/* Emulate ARRAY_ELEM list */
 			zval ch0, ch1;
@@ -625,7 +675,16 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 
 	}
 
-	if (state->version >= 50 && ast_kind_is_decl(ast->kind)) {
+#if PHP_VERSION_ID < 70100
+	/* Emulate docComment on constants, which is not available in PHP 7.0 */
+	if (state->version >= 60 && ast->kind == ZEND_AST_CONST_ELEM) {
+		zval tmp;
+		ZVAL_NULL(&tmp);
+		zend_hash_add_new(ht, AST_STR(str_docComment), &tmp);
+	}
+#endif
+
+	if (ast_kind_is_decl(ast->kind)) {
 		zval id_zval;
 		ZVAL_LONG(&id_zval, state->declIdCounter);
 		state->declIdCounter++;
@@ -635,7 +694,6 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 
 static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 	zval tmp_zv, children_zv;
-	zend_bool is_decl;
 
 	if (ast == NULL) {
 		ZVAL_NULL(zv);
@@ -648,9 +706,11 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 	}
 
 	switch (ast->kind) {
+#if PHP_VERSION_ID < 70400
 		case ZEND_AST_ASSIGN_OP:
 			ast->attr = ast_assign_op_to_binary_op(ast->attr);
 			break;
+#endif
 		case ZEND_AST_GREATER:
 			ast->kind = ZEND_AST_BINARY_OP;
 			ast->attr = AST_BINARY_IS_GREATER;
@@ -668,10 +728,8 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 			ast->attr = AST_BINARY_BOOL_AND;
 			break;
 		case ZEND_AST_COALESCE:
-			if (state->version >= 40) {
-				ast->kind = ZEND_AST_BINARY_OP;
-				ast->attr = AST_BINARY_COALESCE;
-			}
+			ast->kind = ZEND_AST_BINARY_OP;
+			ast->attr = AST_BINARY_COALESCE;
 			break;
 		case ZEND_AST_SILENCE:
 			ast->kind = ZEND_AST_UNARY_OP;
@@ -685,22 +743,81 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 			ast->kind = ZEND_AST_UNARY_OP;
 			ast->attr = AST_MINUS;
 			break;
+#if PHP_VERSION_ID >= 70400
+		case ZEND_AST_PROP_GROUP:
+			if (state->version < 70) {
+				// In versions less than 70, just omit property type information entirely.
+				// ast->child is [type_ast, prop_ast]
+				ast_to_zval(zv, ast->child[1], state);
+				// The property visibility is on the AST_PROP_GROUP node.
+				// Add it to the AST_PROP_DECL node for old
+				ast_update_property_long(zv, AST_STR(str_flags), ast->attr, AST_CACHE_SLOT_FLAGS);
+				return;
+			}
+			break;
+		case ZEND_AST_ASSIGN_COALESCE:
+			ast->kind = ZEND_AST_ASSIGN_OP;
+			ast->attr = AST_BINARY_COALESCE;
+			break;
+		case ZEND_AST_CLASS_NAME:
+			if (state->version < 70) {
+				zval name_zval;
+				ast_to_zval(&name_zval, ast->child[0], state);
+				zval class_name_zval;
+				ast_create_virtual_node_ex(
+						&class_name_zval, AST_NAME, ast->child[0]->attr, zend_ast_get_lineno(ast), state, 1, &name_zval);
+				zval const_zval;
+				ZVAL_STR_COPY(&const_zval, AST_STR(str_class));
+				ast_create_virtual_node_ex(
+						zv, ZEND_AST_CLASS_CONST, 0, zend_ast_get_lineno(ast), state, 2, &class_name_zval, &const_zval);
+				return;
+			}
+			break;
+#ifdef ZEND_PARENTHESIZED_CONCAT
+		case ZEND_AST_BINARY_OP:
+			if (ast->attr == ZEND_PARENTHESIZED_CONCAT) {
+				ast->attr = ZEND_CONCAT;
+			}
+			break;
+#endif
+#else
+		case ZEND_AST_CLASS_CONST:
+			if (state->version >= 70) {
+				// Convert to an AST_CLASS_NAME instead. This is the opposite of the work done in the ZEND_AST_CLASS_NAME case.
+				zend_ast *const_name_ast = ast->child[1];
+				zend_string *str = zend_ast_get_str(const_name_ast);
+				if (zend_string_equals_ci(AST_STR(str_class), str)) {
+					zend_ast *class_name_ast = ast->child[0];
+					zval class_name_zval;
+					if (class_name_ast->kind == ZEND_AST_ZVAL) {
+						// e.g. Foo::class
+						zval class_name_raw_zval;
+						ZVAL_COPY(&class_name_raw_zval, zend_ast_get_zval(class_name_ast));
+						ast_create_virtual_node_ex(
+								&class_name_zval, AST_NAME, class_name_ast->attr, zend_ast_get_lineno(class_name_ast), state, 1, &class_name_raw_zval);
+					} else {
+						// e.g. []::class (not a parse error, but a runtime error)
+						ast_to_zval(&class_name_zval, class_name_ast, state);
+					}
+
+					ast_create_virtual_node_ex(
+							zv, ZEND_AST_CLASS_NAME, 0, zend_ast_get_lineno(ast), state, 1, &class_name_zval);
+					return;
+				}
+			}
+			break;
+#endif
 	}
 
-#if PHP_VERSION_ID >= 70100
-	if (state->version < 35 && ast->kind == ZEND_AST_ARRAY && ast_array_is_list(ast)) {
-		ast->kind = ZEND_AST_LIST;
-		ast->attr = 0;
-	}
-#else
-	if (state->version >= 35 && ast->kind == ZEND_AST_LIST) {
+#if PHP_VERSION_ID < 70100
+	/* Normalize to PHP 7.1 format */
+	if (ast->kind == ZEND_AST_LIST) {
 		ast->kind = ZEND_AST_ARRAY;
 		ast->attr = ZEND_ARRAY_SYNTAX_LIST;
 	}
 #endif
 
-	is_decl = ast_kind_is_decl(ast->kind);
-	object_init_ex(zv, is_decl && state->version < 50 ? ast_decl_ce : ast_node_ce);
+	object_init_ex(zv, ast_node_ce);
 
 	ast_update_property_long(zv, AST_STR(str_kind), ast->kind, AST_CACHE_SLOT_KIND);
 
@@ -710,7 +827,7 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 	Z_DELREF(children_zv);
 	ast_update_property(zv, AST_STR(str_children), &children_zv, AST_CACHE_SLOT_CHILDREN);
 
-	if (is_decl) {
+	if (ast_kind_is_decl(ast->kind)) {
 		zend_ast_decl *decl = (zend_ast_decl *) ast;
 
 		ast_update_property_long(zv, AST_STR(str_flags), decl->flags, AST_CACHE_SLOT_FLAGS);
@@ -722,24 +839,18 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 		} else {
 			ZVAL_NULL(&tmp_zv);
 		}
-		if (state->version < 50) {
-			ast_update_property(zv, AST_STR(str_name), &tmp_zv, NULL);
-		} else {
-			Z_TRY_ADDREF(tmp_zv);
-			zend_hash_add_new(Z_ARRVAL(children_zv), AST_STR(str_name), &tmp_zv);
-		}
+
+		Z_TRY_ADDREF(tmp_zv);
+		zend_hash_add_new(Z_ARRVAL(children_zv), AST_STR(str_name), &tmp_zv);
 
 		if (decl->doc_comment) {
 			ZVAL_STR(&tmp_zv, decl->doc_comment);
 		} else {
 			ZVAL_NULL(&tmp_zv);
 		}
-		if (state->version < 50) {
-			ast_update_property(zv, AST_STR(str_docComment), &tmp_zv, NULL);
-		} else {
-			Z_TRY_ADDREF(tmp_zv);
-			zend_hash_add_new(Z_ARRVAL(children_zv), AST_STR(str_docComment), &tmp_zv);
-		}
+
+		Z_TRY_ADDREF(tmp_zv);
+		zend_hash_add_new(Z_ARRVAL(children_zv), AST_STR(str_docComment), &tmp_zv);
 	} else {
 #if PHP_VERSION_ID < 70100
 		if (ast->kind == ZEND_AST_CLASS_CONST_DECL) {
@@ -749,28 +860,26 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 		ast_update_property_long(zv, AST_STR(str_flags), ast->attr, AST_CACHE_SLOT_FLAGS);
 	}
 
-	if (state->version < 50) {
-		/* Convert doc comments on properties and constants into properties */
-		if (ast->kind == ZEND_AST_PROP_ELEM && ast->child[2]) {
-			ZVAL_STR(&tmp_zv, zend_ast_get_str(ast->child[2]));
-			ast_update_property(zv, AST_STR(str_docComment), &tmp_zv, NULL);
-		}
-#if PHP_VERSION_ID >= 70100
-		if (ast->kind == ZEND_AST_CONST_ELEM && ast->child[2]) {
-			ZVAL_STR(&tmp_zv, zend_ast_get_str(ast->child[2]));
-			ast_update_property(zv, AST_STR(str_docComment), &tmp_zv, NULL);
-		}
-#endif
-	}
-
 	ast_fill_children_ht(Z_ARRVAL(children_zv), ast, state);
+#if PHP_VERSION_ID < 70400
+	if (ast->kind == ZEND_AST_PROP_DECL && state->version >= 70) {
+		zval type_zval;
+		zval prop_group_zval = *zv;
+		ZVAL_NULL(&type_zval);
+		// For version 70, create an AST_PROP_GROUP wrapping the created AST_PROP_DECL.
+		ast_create_virtual_node_ex(
+				zv, ZEND_AST_PROP_GROUP, ast->attr, zend_ast_get_lineno(ast), state, 2, &type_zval, &prop_group_zval);
+		ast_update_property_long(&prop_group_zval, AST_STR(str_flags), 0, AST_CACHE_SLOT_FLAGS);
+	}
+#endif
 }
 
-static const zend_long versions[] = {30, 35, 40, 45, 50};
+static const zend_long versions[] = {50, 60, 70};
 static const size_t versions_count = sizeof(versions)/sizeof(versions[0]);
 
 static inline zend_bool ast_version_deprecated(zend_long version) {
-	return version == 30 || version == 35;
+	/* Currently no deprecated versions */
+	return 0;
 }
 
 static zend_string *ast_version_info() {
@@ -852,13 +961,8 @@ PHP_FUNCTION(parse_file) {
 	zend_restore_error_handling(&error_handling);
 
 	if (!code) {
-		if (version >= 50) {
-			// php_stream_copy_to_mem will return NULL if the file is empty, strangely.
-			// Fix this in new versions, preserve old behavior for version < 50
-			code = ZSTR_EMPTY_ALLOC();
-		} else {
-			return;
-		}
+		/* php_stream_copy_to_mem will return NULL if the file is empty, strangely. */
+		code = ZSTR_EMPTY_ALLOC();
 	}
 
 	ast = get_ast(code, &arena, filename->val);
@@ -914,7 +1018,7 @@ PHP_FUNCTION(get_kind_name) {
 
 	name = ast_kind_to_name(kind);
 	if (!name) {
-		ast_throw_exception(spl_ce_LogicException, "Unknown kind %pd", kind);
+		ast_throw_exception(spl_ce_LogicException, "Unknown kind " ZEND_LONG_FMT, kind);
 		return;
 	}
 
@@ -953,12 +1057,12 @@ static void ast_build_metadata(zval *result) {
 		object_init_ex(&info_zv, ast_metadata_ce);
 
 		/* kind */
-		ast_update_property_long(&info_zv, AST_G(str_kind), kind, NULL);
+		ast_update_property_long(&info_zv, AST_STR(str_kind), kind, NULL);
 
 		/* name */
 		ZVAL_STRING(&tmp_zv, ast_kind_to_name(kind));
 		Z_TRY_DELREF(tmp_zv);
-		ast_update_property(&info_zv, AST_G(str_name), &tmp_zv, NULL);
+		ast_update_property(&info_zv, AST_STR(str_name), &tmp_zv, NULL);
 
 		/* flags */
 		array_init(&tmp_zv);
@@ -969,11 +1073,11 @@ static void ast_build_metadata(zval *result) {
 			}
 		}
 		Z_TRY_DELREF(tmp_zv);
-		ast_update_property(&info_zv, AST_G(str_flags), &tmp_zv, NULL);
+		ast_update_property(&info_zv, AST_STR(str_flags), &tmp_zv, NULL);
 
 		/* flagsCombinable */
 		ZVAL_BOOL(&tmp_zv, flag_info && flag_info->combinable);
-		ast_update_property(&info_zv, AST_G(str_flagsCombinable), &tmp_zv, NULL);
+		ast_update_property(&info_zv, AST_STR(str_flagsCombinable), &tmp_zv, NULL);
 
 		add_index_zval(result, kind, &info_zv);
 	}
@@ -1057,12 +1161,12 @@ PHP_METHOD(ast_Node, __construct) {
 	}
 }
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_parse_file, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_parse_file, 0, 0, 2)
 	ZEND_ARG_INFO(0, filename)
 	ZEND_ARG_INFO(0, version)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_parse_code, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_parse_code, 0, 0, 2)
 	ZEND_ARG_INFO(0, code)
 	ZEND_ARG_INFO(0, version)
 	ZEND_ARG_INFO(0, filename)
@@ -1101,14 +1205,20 @@ const zend_function_entry ast_functions[] = {
 };
 
 const zend_function_entry ast_node_functions[] = {
-	PHP_ME(ast_Node, __construct, arginfo_node_construct, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
+	PHP_ME(ast_Node, __construct, arginfo_node_construct, ZEND_ACC_PUBLIC)
 	PHP_FE_END
 };
 
 PHP_MINFO_FUNCTION(ast) {
+	zend_string *info = ast_version_info();
+
 	php_info_print_table_start();
 	php_info_print_table_row(2, "ast support", "enabled");
+	php_info_print_table_row(2, "extension version", PHP_AST_VERSION);
+	php_info_print_table_row(2, "AST version", ZSTR_VAL(info));
 	php_info_print_table_end();
+
+	zend_string_release(info);
 }
 
 PHP_RINIT_FUNCTION(ast) {
@@ -1150,7 +1260,8 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("FUNC_RETURNS_REF", ZEND_ACC_RETURN_REFERENCE);
 	ast_register_flag_constant("FUNC_GENERATOR", ZEND_ACC_GENERATOR);
 
-	ast_register_flag_constant("CLOSURE_USE_REF", 1);
+	ast_register_flag_constant("ARRAY_ELEM_REF", 1);
+	ast_register_flag_constant("CLOSURE_USE_REF", ZEND_BIND_REF);
 
 	ast_register_flag_constant("CLASS_ABSTRACT", ZEND_ACC_EXPLICIT_ABSTRACT_CLASS);
 	ast_register_flag_constant("CLASS_FINAL", ZEND_ACC_FINAL);
@@ -1204,19 +1315,6 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("BINARY_SPACESHIP", ZEND_SPACESHIP);
 	ast_register_flag_constant("BINARY_COALESCE", AST_BINARY_COALESCE);
 
-	ast_register_flag_constant("ASSIGN_BITWISE_OR", ZEND_ASSIGN_BW_OR);
-	ast_register_flag_constant("ASSIGN_BITWISE_AND", ZEND_ASSIGN_BW_AND);
-	ast_register_flag_constant("ASSIGN_BITWISE_XOR", ZEND_ASSIGN_BW_XOR);
-	ast_register_flag_constant("ASSIGN_CONCAT", ZEND_ASSIGN_CONCAT);
-	ast_register_flag_constant("ASSIGN_ADD", ZEND_ASSIGN_ADD);
-	ast_register_flag_constant("ASSIGN_SUB", ZEND_ASSIGN_SUB);
-	ast_register_flag_constant("ASSIGN_MUL", ZEND_ASSIGN_MUL);
-	ast_register_flag_constant("ASSIGN_DIV", ZEND_ASSIGN_DIV);
-	ast_register_flag_constant("ASSIGN_MOD", ZEND_ASSIGN_MOD);
-	ast_register_flag_constant("ASSIGN_POW", ZEND_ASSIGN_POW);
-	ast_register_flag_constant("ASSIGN_SHIFT_LEFT", ZEND_ASSIGN_SL);
-	ast_register_flag_constant("ASSIGN_SHIFT_RIGHT", ZEND_ASSIGN_SR);
-
 	ast_register_flag_constant("EXEC_EVAL", ZEND_EVAL);
 	ast_register_flag_constant("EXEC_INCLUDE", ZEND_INCLUDE);
 	ast_register_flag_constant("EXEC_INCLUDE_ONCE", ZEND_INCLUDE_ONCE);
@@ -1246,6 +1344,10 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("ARRAY_SYNTAX_LONG", ZEND_ARRAY_SYNTAX_LONG);
 	ast_register_flag_constant("ARRAY_SYNTAX_SHORT", ZEND_ARRAY_SYNTAX_SHORT);
 
+	ast_register_flag_constant("DIM_ALTERNATIVE_SYNTAX", ZEND_DIM_ALTERNATIVE_SYNTAX);
+
+	ast_register_flag_constant("PARENTHESIZED_CONDITIONAL", ZEND_PARENTHESIZED_CONDITIONAL);
+
 	INIT_CLASS_ENTRY(tmp_ce, "ast\\Node", ast_node_functions);
 	ast_node_ce = zend_register_internal_class(&tmp_ce);
 	ast_declare_property(ast_node_ce, AST_STR(str_kind), &zv_null);
@@ -1253,12 +1355,6 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_declare_property(ast_node_ce, AST_STR(str_lineno), &zv_null);
 	ast_declare_property(ast_node_ce, AST_STR(str_children), &zv_null);
 
-	INIT_CLASS_ENTRY(tmp_ce, "ast\\Node\\Decl", NULL);
-	ast_decl_ce = zend_register_internal_class_ex(&tmp_ce, ast_node_ce);
-	ast_declare_property(ast_decl_ce, AST_STR(str_endLineno), &zv_null);
-	ast_declare_property(ast_decl_ce, AST_STR(str_name), &zv_null);
-	ast_declare_property(ast_decl_ce, AST_STR(str_docComment), &zv_null);
-
 	INIT_CLASS_ENTRY(tmp_ce, "ast\\Metadata", NULL);
 	ast_metadata_ce = zend_register_internal_class(&tmp_ce);
 	ast_declare_property(ast_metadata_ce, AST_STR(str_kind), &zv_null);
