@@ -1,7 +1,5 @@
 #include "php_ast.h"
 
-const size_t ast_kinds_count = 99;
-
 const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_ARG_LIST,
 	ZEND_AST_LIST,
@@ -26,18 +24,16 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_FUNC_DECL,
 	ZEND_AST_CLOSURE,
 	ZEND_AST_METHOD,
+	ZEND_AST_ARROW_FUNC,
 	ZEND_AST_CLASS,
 	ZEND_AST_MAGIC_CONST,
 	ZEND_AST_TYPE,
 	ZEND_AST_VAR,
 	ZEND_AST_CONST,
 	ZEND_AST_UNPACK,
-	ZEND_AST_UNARY_PLUS,
-	ZEND_AST_UNARY_MINUS,
 	ZEND_AST_CAST,
 	ZEND_AST_EMPTY,
 	ZEND_AST_ISSET,
-	ZEND_AST_SILENCE,
 	ZEND_AST_SHELL_EXEC,
 	ZEND_AST_CLONE,
 	ZEND_AST_EXIT,
@@ -60,6 +56,7 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_GOTO,
 	ZEND_AST_BREAK,
 	ZEND_AST_CONTINUE,
+	ZEND_AST_CLASS_NAME,
 	ZEND_AST_DIM,
 	ZEND_AST_PROP,
 	ZEND_AST_STATIC_PROP,
@@ -69,15 +66,10 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_ASSIGN_REF,
 	ZEND_AST_ASSIGN_OP,
 	ZEND_AST_BINARY_OP,
-	ZEND_AST_GREATER,
-	ZEND_AST_GREATER_EQUAL,
-	ZEND_AST_AND,
-	ZEND_AST_OR,
 	ZEND_AST_ARRAY_ELEM,
 	ZEND_AST_NEW,
 	ZEND_AST_INSTANCEOF,
 	ZEND_AST_YIELD,
-	ZEND_AST_COALESCE,
 	ZEND_AST_STATIC,
 	ZEND_AST_WHILE,
 	ZEND_AST_DO_WHILE,
@@ -86,6 +78,7 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_SWITCH_CASE,
 	ZEND_AST_DECLARE,
 	ZEND_AST_PROP_ELEM,
+	ZEND_AST_PROP_GROUP,
 	ZEND_AST_CONST_ELEM,
 	ZEND_AST_USE_TRAIT,
 	ZEND_AST_TRAIT_PRECEDENCE,
@@ -104,6 +97,8 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_FOREACH,
 };
 
+const size_t ast_kinds_count = sizeof(ast_kinds) / sizeof(ast_kinds[0]);
+
 const char *ast_kind_to_name(zend_ast_kind kind) {
 	switch (kind) {
 		case ZEND_AST_ARG_LIST: return "AST_ARG_LIST";
@@ -129,18 +124,16 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_FUNC_DECL: return "AST_FUNC_DECL";
 		case ZEND_AST_CLOSURE: return "AST_CLOSURE";
 		case ZEND_AST_METHOD: return "AST_METHOD";
+		case ZEND_AST_ARROW_FUNC: return "AST_ARROW_FUNC";
 		case ZEND_AST_CLASS: return "AST_CLASS";
 		case ZEND_AST_MAGIC_CONST: return "AST_MAGIC_CONST";
 		case ZEND_AST_TYPE: return "AST_TYPE";
 		case ZEND_AST_VAR: return "AST_VAR";
 		case ZEND_AST_CONST: return "AST_CONST";
 		case ZEND_AST_UNPACK: return "AST_UNPACK";
-		case ZEND_AST_UNARY_PLUS: return "AST_UNARY_PLUS";
-		case ZEND_AST_UNARY_MINUS: return "AST_UNARY_MINUS";
 		case ZEND_AST_CAST: return "AST_CAST";
 		case ZEND_AST_EMPTY: return "AST_EMPTY";
 		case ZEND_AST_ISSET: return "AST_ISSET";
-		case ZEND_AST_SILENCE: return "AST_SILENCE";
 		case ZEND_AST_SHELL_EXEC: return "AST_SHELL_EXEC";
 		case ZEND_AST_CLONE: return "AST_CLONE";
 		case ZEND_AST_EXIT: return "AST_EXIT";
@@ -163,6 +156,7 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_GOTO: return "AST_GOTO";
 		case ZEND_AST_BREAK: return "AST_BREAK";
 		case ZEND_AST_CONTINUE: return "AST_CONTINUE";
+		case ZEND_AST_CLASS_NAME: return "AST_CLASS_NAME";
 		case ZEND_AST_DIM: return "AST_DIM";
 		case ZEND_AST_PROP: return "AST_PROP";
 		case ZEND_AST_STATIC_PROP: return "AST_STATIC_PROP";
@@ -172,15 +166,10 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_ASSIGN_REF: return "AST_ASSIGN_REF";
 		case ZEND_AST_ASSIGN_OP: return "AST_ASSIGN_OP";
 		case ZEND_AST_BINARY_OP: return "AST_BINARY_OP";
-		case ZEND_AST_GREATER: return "AST_GREATER";
-		case ZEND_AST_GREATER_EQUAL: return "AST_GREATER_EQUAL";
-		case ZEND_AST_AND: return "AST_AND";
-		case ZEND_AST_OR: return "AST_OR";
 		case ZEND_AST_ARRAY_ELEM: return "AST_ARRAY_ELEM";
 		case ZEND_AST_NEW: return "AST_NEW";
 		case ZEND_AST_INSTANCEOF: return "AST_INSTANCEOF";
 		case ZEND_AST_YIELD: return "AST_YIELD";
-		case ZEND_AST_COALESCE: return "AST_COALESCE";
 		case ZEND_AST_STATIC: return "AST_STATIC";
 		case ZEND_AST_WHILE: return "AST_WHILE";
 		case ZEND_AST_DO_WHILE: return "AST_DO_WHILE";
@@ -189,6 +178,7 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_SWITCH_CASE: return "AST_SWITCH_CASE";
 		case ZEND_AST_DECLARE: return "AST_DECLARE";
 		case ZEND_AST_PROP_ELEM: return "AST_PROP_ELEM";
+		case ZEND_AST_PROP_GROUP: return "AST_PROP_GROUP";
 		case ZEND_AST_CONST_ELEM: return "AST_CONST_ELEM";
 		case ZEND_AST_USE_TRAIT: return "AST_USE_TRAIT";
 		case ZEND_AST_TRAIT_PRECEDENCE: return "AST_TRAIT_PRECEDENCE";
@@ -251,6 +241,14 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 3: return AST_STR(str_returnType);
 			}
 			return NULL;
+		case ZEND_AST_ARROW_FUNC:
+			switch (child) {
+				case 0: return AST_STR(str_params);
+				case 1: return AST_STR(str_uses);
+				case 2: return AST_STR(str_stmts);
+				case 3: return AST_STR(str_returnType);
+			}
+			return NULL;
 		case ZEND_AST_CLASS:
 			switch (child) {
 				case 0: return AST_STR(str_extends);
@@ -277,16 +275,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 0: return AST_STR(str_expr);
 			}
 			return NULL;
-		case ZEND_AST_UNARY_PLUS:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_UNARY_MINUS:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_CAST:
 			switch (child) {
 				case 0: return AST_STR(str_expr);
@@ -302,11 +290,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 0: return AST_STR(str_var);
 			}
 			return NULL;
-		case ZEND_AST_SILENCE:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_SHELL_EXEC:
 			switch (child) {
 				case 0: return AST_STR(str_expr);
@@ -417,6 +400,11 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 0: return AST_STR(str_depth);
 			}
 			return NULL;
+		case ZEND_AST_CLASS_NAME:
+			switch (child) {
+				case 0: return AST_STR(str_class);
+			}
+			return NULL;
 		case ZEND_AST_DIM:
 			switch (child) {
 				case 0: return AST_STR(str_expr);
@@ -471,30 +459,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 1: return AST_STR(str_right);
 			}
 			return NULL;
-		case ZEND_AST_GREATER:
-			switch (child) {
-				case 0: return AST_STR(str_left);
-				case 1: return AST_STR(str_right);
-			}
-			return NULL;
-		case ZEND_AST_GREATER_EQUAL:
-			switch (child) {
-				case 0: return AST_STR(str_left);
-				case 1: return AST_STR(str_right);
-			}
-			return NULL;
-		case ZEND_AST_AND:
-			switch (child) {
-				case 0: return AST_STR(str_left);
-				case 1: return AST_STR(str_right);
-			}
-			return NULL;
-		case ZEND_AST_OR:
-			switch (child) {
-				case 0: return AST_STR(str_left);
-				case 1: return AST_STR(str_right);
-			}
-			return NULL;
 		case ZEND_AST_ARRAY_ELEM:
 			switch (child) {
 				case 0: return AST_STR(str_value);
@@ -519,12 +483,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 1: return AST_STR(str_key);
 			}
 			return NULL;
-		case ZEND_AST_COALESCE:
-			switch (child) {
-				case 0: return AST_STR(str_left);
-				case 1: return AST_STR(str_right);
-			}
-			return NULL;
 		case ZEND_AST_STATIC:
 			switch (child) {
 				case 0: return AST_STR(str_var);
@@ -574,6 +532,12 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 2: return AST_STR(str_docComment);
 			}
 			return NULL;
+		case ZEND_AST_PROP_GROUP:
+			switch (child) {
+				case 0: return AST_STR(str_type);
+				case 1: return AST_STR(str_props);
+			}
+			return NULL;
 		case ZEND_AST_CONST_ELEM:
 			switch (child) {
 				case 0: return AST_STR(str_name);
@@ -710,18 +674,16 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_FUNC_DECL", ZEND_AST_FUNC_DECL, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLOSURE", ZEND_AST_CLOSURE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_METHOD", ZEND_AST_METHOD, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_ARROW_FUNC", ZEND_AST_ARROW_FUNC, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLASS", ZEND_AST_CLASS, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_MAGIC_CONST", ZEND_AST_MAGIC_CONST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TYPE", ZEND_AST_TYPE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_VAR", ZEND_AST_VAR, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CONST", ZEND_AST_CONST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_UNPACK", ZEND_AST_UNPACK, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_UNARY_PLUS", ZEND_AST_UNARY_PLUS, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_UNARY_MINUS", ZEND_AST_UNARY_MINUS, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CAST", ZEND_AST_CAST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_EMPTY", ZEND_AST_EMPTY, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ISSET", ZEND_AST_ISSET, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_SILENCE", ZEND_AST_SILENCE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_SHELL_EXEC", ZEND_AST_SHELL_EXEC, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLONE", ZEND_AST_CLONE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_EXIT", ZEND_AST_EXIT, CONST_CS | CONST_PERSISTENT);
@@ -744,6 +706,7 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_GOTO", ZEND_AST_GOTO, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_BREAK", ZEND_AST_BREAK, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CONTINUE", ZEND_AST_CONTINUE, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLASS_NAME", ZEND_AST_CLASS_NAME, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_DIM", ZEND_AST_DIM, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_PROP", ZEND_AST_PROP, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_STATIC_PROP", ZEND_AST_STATIC_PROP, CONST_CS | CONST_PERSISTENT);
@@ -753,15 +716,10 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ASSIGN_REF", ZEND_AST_ASSIGN_REF, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ASSIGN_OP", ZEND_AST_ASSIGN_OP, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_BINARY_OP", ZEND_AST_BINARY_OP, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_GREATER", ZEND_AST_GREATER, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_GREATER_EQUAL", ZEND_AST_GREATER_EQUAL, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_AND", ZEND_AST_AND, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_OR", ZEND_AST_OR, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ARRAY_ELEM", ZEND_AST_ARRAY_ELEM, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_NEW", ZEND_AST_NEW, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_INSTANCEOF", ZEND_AST_INSTANCEOF, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_YIELD", ZEND_AST_YIELD, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_COALESCE", ZEND_AST_COALESCE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_STATIC", ZEND_AST_STATIC, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_WHILE", ZEND_AST_WHILE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_DO_WHILE", ZEND_AST_DO_WHILE, CONST_CS | CONST_PERSISTENT);
@@ -770,6 +728,7 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_SWITCH_CASE", ZEND_AST_SWITCH_CASE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_DECLARE", ZEND_AST_DECLARE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_PROP_ELEM", ZEND_AST_PROP_ELEM, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_PROP_GROUP", ZEND_AST_PROP_GROUP, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CONST_ELEM", ZEND_AST_CONST_ELEM, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_USE_TRAIT", ZEND_AST_USE_TRAIT, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TRAIT_PRECEDENCE", ZEND_AST_TRAIT_PRECEDENCE, CONST_CS | CONST_PERSISTENT);
