@@ -7,7 +7,7 @@
 extern zend_module_entry ast_module_entry;
 #define phpext_ast_ptr &ast_module_entry
 
-#define PHP_AST_VERSION "0.1.6"
+#define PHP_AST_VERSION "1.0.3"
 
 #ifdef PHP_WIN32
 #	define PHP_AST_API __declspec(dllexport)
@@ -21,12 +21,11 @@ extern zend_module_entry ast_module_entry;
 #include "TSRM.h"
 #endif
 
-#define AST_NUM_CACHE_SLOTS (2 * 4)
+// PHP 7.4 added a 3rd cache slot for property_info
+// and expects cache_slot[2] to be null.
+#define AST_NUM_CACHE_SLOTS (3 * 4)
 
 ZEND_BEGIN_MODULE_GLOBALS(ast)
-#define X(str) zend_string *str_ ## str;
-	AST_STR_DEFS
-#undef X
 	void *cache_slots[AST_NUM_CACHE_SLOTS];
 	zval metadata;
 ZEND_END_MODULE_GLOBALS(ast)
@@ -35,13 +34,29 @@ ZEND_EXTERN_MODULE_GLOBALS(ast)
 
 #define AST_G(v) ZEND_MODULE_GLOBALS_ACCESSOR(ast, v)
 
-#define AST_STR(str) AST_G(str)
+typedef struct _ast_str_globals {
+#define X(str) zend_string *str_ ## str;
+	AST_STR_DEFS
+#undef X
+} ast_str_globals;
+
+extern ast_str_globals str_globals;
+
+#define AST_STR(str) str_globals.str
 
 /* Custom ast kind for names */
 #define AST_NAME          2048
 #define AST_CLOSURE_VAR   2049
 #define AST_NULLABLE_TYPE 2050
 
+// 544 is already taken by ZEND_AST_GROUP_USE
+#if PHP_VERSION_ID < 70400
+// NOTE: The first hex digit is the number of child nodes a given kind has
+# define ZEND_AST_CLASS_NAME 0x1ff
+# define ZEND_AST_PROP_GROUP 0x2ff
+# define ZEND_AST_ARROW_FUNC 0x5ff
+#endif
+
 /* Pretend it still exists */
 #if PHP_VERSION_ID >= 70100
 # define ZEND_AST_LIST ((1 << (ZEND_AST_IS_LIST_SHIFT + 1)) - 1)
